/* ip_qos_attr_def_support.ex.c 							*/
/* Attribute definiton procedures for the IP QoS model.		*/
/* In the abscence of the IP QoS Configuration object		*/
/* attributes for 'Queuing Profiles' and 'CAR Parameters	*/
/* are set to their default values.							*/
/****************************************/
/*	Copyright (c) 1987 - 1999			*/
/*			by MIL 3, Inc.				*/
/*		(A Delaware Corporation)		*/
/*	3400 International Drive,  N.W.		*/
/*		Washington, D.C., U.S.A.		*/
/*			All Rights Reserved.		*/
/****************************************/

#include <opnet.h>
#include <oms_qm.h>
#include <oms_pr.h>
#include <oms_data_def.h>
#include <ip_rte_sup_v4.h>
#include <ip_notif_log_support.h>
#include <ip_qos_notif_log_support.h>
#include <string.h>

/****** General function to see if global attribute definer object exists. ******/

void
ip_rte_qos_attr_config_info ()
	{
	List*			proc_record_handle_list_ptr;
	static Boolean	ip_attribute_object_exists = OPC_FALSE;

	/** Check the existence of ip attribute object. We will read the compression	**/
	/** information for each interface from the attribute database. The special 	**/
	/** object defines all the compession informaiton. Ensure that this code		**/
	/** check is not repeated.														**/
	FIN (ip_rte_qos_attr_config_info ());

	if (ip_attribute_object_exists == OPC_FALSE)
		{
		/* Turn on the global flag so that no other IP module repeats this		*/
		/* process.																*/
		ip_attribute_object_exists = OPC_TRUE;

		/* Check the existence of attribute object using process registry. If 	*/
		/* the object exists then it has already registered itself and set the	*/
		/* protocol attribute to "ip_attribute_object".							*/ 
		proc_record_handle_list_ptr = op_prg_list_create ();
		oms_pr_process_discover (OPC_OBJID_INVALID, proc_record_handle_list_ptr,
							 "protocol", OMSC_PR_STRING, "ip_qos_attribute_object",
							 OPC_NIL);
		if (op_prg_list_size (proc_record_handle_list_ptr) == 0)
			{
			/* The special IP attribute object is missing. Take its				*/
			/* responsibility and at least register the defaults into the 		*/
			/* attribute database for the attributes that would be registered	*/
			/* by this missing object.											*/
			ip_rte_queuing_profiles_defaults_register ();	
		 	ip_rte_car_profiles_defaults_register ();	

			/* Writes a simulation log about the absence of the configuration object.	*/
			qosnl_confwarn_missing_conf_obj ();
			}

		/* Deallocate memory occupied by process discovery information.			*/
		while (op_prg_list_size (proc_record_handle_list_ptr))
			op_prg_list_remove (proc_record_handle_list_ptr, OPC_LISTPOS_HEAD);
		op_prg_mem_free (proc_record_handle_list_ptr);
		}

	FOUT;
	}


/****** Queuing-related attribute description. ******/

void
ip_rte_queuing_profiles_defaults_register ()
	{
	OmsT_Qm_Attributes*					temp_qm_info_ptr = OPC_NIL;
	int									queue;
	OmsT_Qm_Classification_Criteria*	classify_scheme;

	/* This function contains various hard-coded values for the		*/
	/* "Queuing Profiles" compound attribute. Normally the values	*/
	/* for this attribute are set in the IP attribute object. That	*/
	/* object parses them, and inserts (registers) the information	*/
	/* into the network-wide attribute database. This function is 	*/
	/* called when this attribute object is absent in the network	*/
	/* model to - at least - insert some commonly used values to	*/
	/* the database so that if only these values are used the		*/
	/* simulation can still continue in spite of non-existence of	*/
	/* IP attribute object.											*/ 
	FIN (ip_rte_queuing_profiles_defaults_register ());

	/* For each commonly used value of the attribute "Queuing Profiles"	*/
	/* do the following:										 		*/
	/* a) Allocate memory for the data structure. The data			 	*/
	/*    structure is declared in the file oms_data_def_ds_defs.h	 	*/
	/*    which is included by oms_data_def.h.						 	*/
	/* b) Fill the fields with hard-coded values.					 	*/
	/* c) Insert the information into attribute database.			 	*/

	/* Enter attributes for ToS Based Queuing */	
	
	/* Allocate memory and define general attributes for one interface */
	temp_qm_info_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));
	temp_qm_info_ptr->name = (char *) op_prg_mem_alloc ((strlen ("ToS Based") + 1) * sizeof (char));
	strcpy (temp_qm_info_ptr->name, "ToS Based");
	temp_qm_info_ptr->no_queues = 8;
	temp_qm_info_ptr->max_total_no_buf_pkts = 500;

	/* Allocate memory for the specific queue attributes */	
	temp_qm_info_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration**) op_prg_mem_alloc (8 * sizeof (OmsT_Qm_Queue_Configuration*));

	for (queue = 0; queue < 8; queue ++)
		{
		temp_qm_info_ptr->queue_configuration[queue] = (OmsT_Qm_Queue_Configuration*) 
			op_prg_mem_alloc (8 * sizeof (OmsT_Qm_Queue_Configuration));
		}

	/* Create the list of criteria.	*/
	temp_qm_info_ptr->classification_list_ptr = op_prg_list_create ();

	/* Loop through the queue and define attributes for each queue */
	for (queue = 0; queue < 8; queue++)
		{
		temp_qm_info_ptr->queue_configuration[queue]->weight = 10 * queue + 10;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 2000 * queue + 2000;
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->tos = queue;
		classify_scheme->protocol = -1;
		classify_scheme->source_port = -1;
		classify_scheme->dest_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->queue_id = queue;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;
		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);
		}

	/* Register the pointer to these attributes in the database */
 	oms_data_def_entry_insert ("WFQ Profiles", "ToS Based", temp_qm_info_ptr);
  	oms_data_def_entry_insert ("Priority Queuing Profiles", "ToS Based", temp_qm_info_ptr);
   	oms_data_def_entry_insert ("Custom Queuing Profiles", "ToS Based", temp_qm_info_ptr);
 

	/* Enter attributes for Protocol Based Queuing */

	/* Allocate memory and define general attributes for one interface */
	temp_qm_info_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));
	temp_qm_info_ptr->name = (char *) op_prg_mem_alloc ((strlen ("Protocol Based") + 1) * sizeof (char));
	strcpy (temp_qm_info_ptr->name, "Protocol Based");
	temp_qm_info_ptr->no_queues = 4;
	temp_qm_info_ptr->max_total_no_buf_pkts = 500;
	
	/* Allocate memory for the specific queue attributes */
	temp_qm_info_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration**) op_prg_mem_alloc (4 * sizeof (OmsT_Qm_Queue_Configuration*));

	for (queue = 0; queue < 4; queue ++)
		{
		temp_qm_info_ptr->queue_configuration[queue] = 
			(OmsT_Qm_Queue_Configuration*) op_prg_mem_alloc (4 * sizeof (OmsT_Qm_Queue_Configuration));
		}

	/* Create the list of criteria. */
	temp_qm_info_ptr->classification_list_ptr = op_prg_list_create ();

	/* Loop through the queue and define attributes for each queue */
	for (queue = 0; queue < 4; queue++)
        {
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 5000;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->queue_id = queue;
		classify_scheme->tos = -1;
		classify_scheme->source_port = -1;
		classify_scheme->dest_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;	
		switch (queue)
			{
			case 0: classify_scheme->protocol = 88;
					break;
			case 1: classify_scheme->protocol = 81;
					break;
			case 2: classify_scheme->protocol = 17;
					break;
			case 3: classify_scheme->protocol = 6;
					break;
			}

		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);
		}

	/* Define the attributes for the weight.		*/
	temp_qm_info_ptr->queue_configuration[0]->weight = 10;
	temp_qm_info_ptr->queue_configuration[1]->weight = 10;
	temp_qm_info_ptr->queue_configuration[2]->weight = 50;
	temp_qm_info_ptr->queue_configuration[3]->weight = 100;

	/* Register the pointer to these attributes in the database */
    oms_data_def_entry_insert ("WFQ Profiles", "Protocol Based", temp_qm_info_ptr);
	oms_data_def_entry_insert ("Priority Queuing Profiles", "Protocol Based", temp_qm_info_ptr);
	oms_data_def_entry_insert ("Custom Queuing Profiles", "Protocol Based", temp_qm_info_ptr);
 
	
	
	
	
	

	
	/* Enter attributes for Address Based Queuing */	

	/* Allocate memory and define general attributes for one interface */
	temp_qm_info_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));
	temp_qm_info_ptr->name = (char *) op_prg_mem_alloc ((strlen ("Address Based") + 1) * sizeof (char));
	strcpy (temp_qm_info_ptr->name, "Address Based");
	temp_qm_info_ptr->no_queues = 4;
	temp_qm_info_ptr->max_total_no_buf_pkts = 500;

	/* Allocate memory for the specific queue attributes */	
	temp_qm_info_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration**) op_prg_mem_alloc (4 * sizeof (OmsT_Qm_Queue_Configuration*));

	for (queue = 0; queue < 4; queue ++)
		{
		temp_qm_info_ptr->queue_configuration[queue] = (OmsT_Qm_Queue_Configuration*) op_prg_mem_alloc (4 * sizeof (OmsT_Qm_Queue_Configuration));
		}
	/* Create the list of criteria. */
	temp_qm_info_ptr->classification_list_ptr = op_prg_list_create ();

	/* Loop through the queue and define attributes for each queue */
	for (queue = 0; queue < 4; queue++)
		{
		  /* Register first classification scheme for source port.*/
		temp_qm_info_ptr->queue_configuration[queue]->weight = 1;
		temp_qm_info_ptr->queue_configuration[queue]->arrival_rate = 1;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 5000;
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->queue_id = queue;
		classify_scheme->tos = -1;
		classify_scheme->protocol = -1;
		classify_scheme->dest_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;	
	
		switch (queue)
			{
			case 0: classify_scheme->source_address = 1;
					break;
			case 1: classify_scheme->source_address = 2; 
					break;
			case 2: classify_scheme->source_address = 3; 
					break;
			case 3: classify_scheme->source_address = 4; 
					break;
			
			}

		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);

		/* Register second classification scheme for destination port.*/
		temp_qm_info_ptr->queue_configuration[queue]->weight = 1;
		temp_qm_info_ptr->queue_configuration[queue]->arrival_rate = 1;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 5000;
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->queue_id = queue;
		classify_scheme->tos = -1;
		classify_scheme->protocol = -1;
		classify_scheme->source_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;	
	
		switch (queue)
			{
			case 0: classify_scheme->dest_address = 5;
					break;
			case 1: classify_scheme->dest_address = 6; 
					break;
			case 2: classify_scheme->dest_address = 7; 
					break;
			case 3: classify_scheme->dest_address = 8; 
					break;
			
			}

		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);
		}

	/* Register the pointer to these attributes in the database */
    
	oms_data_def_entry_insert ("VC Queuing Profiles", "Address Based", temp_qm_info_ptr);

	
	
	
	
	
	
	
	/* Enter attributes for Port Based Queuing */	

	/* Allocate memory and define general attributes for one interface */
	temp_qm_info_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));
	temp_qm_info_ptr->name = (char *) op_prg_mem_alloc ((strlen ("Port Based") + 1) * sizeof (char));
	strcpy (temp_qm_info_ptr->name, "Port Based");
	temp_qm_info_ptr->no_queues = 9;
	temp_qm_info_ptr->max_total_no_buf_pkts = 500;

	/* Allocate memory for the specific queue attributes */	
	temp_qm_info_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration**) op_prg_mem_alloc (9 * sizeof (OmsT_Qm_Queue_Configuration*));

	for (queue = 0; queue < 9; queue ++)
		{
		temp_qm_info_ptr->queue_configuration[queue] = (OmsT_Qm_Queue_Configuration*) op_prg_mem_alloc (9 * sizeof (OmsT_Qm_Queue_Configuration));
		}
	/* Create the list of criteria. */
	temp_qm_info_ptr->classification_list_ptr = op_prg_list_create ();

	/* Loop through the queue and define attributes for each queue */
	for (queue = 0; queue < 9; queue++)
		{
		  /* Register first classification scheme for source port.*/
		temp_qm_info_ptr->queue_configuration[queue]->weight = 1;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 5000;
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->queue_id = queue;
		classify_scheme->tos = -1;
		classify_scheme->protocol = -1;
		classify_scheme->dest_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;	
	
		switch (queue)
			{
			case 0: classify_scheme->source_port = 23;
					break;
			case 1: classify_scheme->source_port = 20; 
					break;
			case 2: classify_scheme->source_port = 25; 
					break;
			case 3: classify_scheme->source_port = 80; 
					break;
			case 4: classify_scheme->source_port = 102; 
					break;
			case 5: classify_scheme->source_port = 105; 
					break;
			case 6: classify_scheme->source_port = 101; 
					break;
			case 7: classify_scheme->source_port = 100; 
					break;
			case 8: classify_scheme->source_port = 104; 
					break;
			}

		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);

		/* Register second classification scheme for destination port.*/
		temp_qm_info_ptr->queue_configuration[queue]->weight = 1;
		temp_qm_info_ptr->queue_configuration[queue]->red_status = 0;
		temp_qm_info_ptr->queue_configuration[queue]->byte_count_number = 5000;
		temp_qm_info_ptr->queue_configuration[queue]->max_size = 500;
		classify_scheme = (OmsT_Qm_Classification_Criteria*) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));
		classify_scheme->queue_id = queue;
		classify_scheme->tos = -1;
		classify_scheme->protocol = -1;
		classify_scheme->source_port = -1;
		classify_scheme->source_address = IPC_ADDR_INVALID;
		classify_scheme->dest_address = IPC_ADDR_INVALID;
		classify_scheme->incoming_iface = -1;	
	
		switch (queue)
			{
			case 0: classify_scheme->dest_port = 23;
					break;
			case 1: classify_scheme->dest_port = 20; 
					break;
			case 2: classify_scheme->dest_port = 25; 
					break;
			case 3: classify_scheme->dest_port = 80; 
					break;
			case 4: classify_scheme->dest_port = 102; 
					break;
			case 5: classify_scheme->dest_port = 105; 
					break;
			case 6: classify_scheme->dest_port = 101; 
					break;
			case 7: classify_scheme->dest_port = 100; 
					break;
			case 8: classify_scheme->dest_port = 104; 
					break;
			}

		op_prg_list_insert (temp_qm_info_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);
		}

	/* Register the pointer to these attributes in the database */
    oms_data_def_entry_insert ("WFQ Profiles", "Port Based", temp_qm_info_ptr);
	oms_data_def_entry_insert ("Priority Queuing Profiles", "Port Based", temp_qm_info_ptr);
	oms_data_def_entry_insert ("Custom Queuing Profiles", "Port Based", temp_qm_info_ptr);
	oms_data_def_entry_insert ("VC Queuing Profiles", "Port Based", temp_qm_info_ptr);

	/* Enter attributes for 'FIFO Profiles' */	

	/* Allocate memory and define general attributes for one interface */
	temp_qm_info_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));
	temp_qm_info_ptr->name = (char *) op_prg_mem_alloc ((strlen ("FIFO Profile") + 1) * sizeof (char));
	strcpy (temp_qm_info_ptr->name, "FIFO Profile");
	temp_qm_info_ptr->no_queues = 1;
	temp_qm_info_ptr->max_total_no_buf_pkts = 500;

	/* Allocate memory for the specific queue attributes */	
	temp_qm_info_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration**) op_prg_mem_alloc (sizeof (OmsT_Qm_Queue_Configuration*));
	temp_qm_info_ptr->queue_configuration[0] = (OmsT_Qm_Queue_Configuration*) op_prg_mem_alloc (sizeof (OmsT_Qm_Queue_Configuration));

	/* Create the list of criteria. */
	temp_qm_info_ptr->classification_list_ptr = op_prg_list_create ();

       	temp_qm_info_ptr->queue_configuration[0]->weight = 1;
       	temp_qm_info_ptr->queue_configuration[0]->byte_count_number = 5000;
       	temp_qm_info_ptr->queue_configuration[0]->max_size = 100;
		temp_qm_info_ptr->queue_configuration[0]->red_status = 0;

	/* Register the pointer to these attributes in the database */
	oms_data_def_entry_insert ("FIFO Profiles", "FIFO Profile", temp_qm_info_ptr);

	FOUT;
	}

	
/****** CAR-related attribute description. ******/

void
ip_rte_car_profiles_defaults_register (void)
	{
	int                     cos_index;
	OmsT_Qm_Car_Profile*    profile_ptr;

	/* This function contains various hard-coded values for the     */
	/* "CAR Profiles" compound attribute. Normally the values		*/
	/* for this attribute are set in the IP attribute object. That  */
	/* object parses them, and inserts (registers) the information  */
	/* into the network-wide attribute database. This function is   */
	/* called when this attribute object is absent in the network   */
	/* model to - at least - insert some commonly used values to    */
	/* the database so that if only these values are used the       */
	/* simulation can still continue in spite of non-existence of   */
	/* IP attribute object.                                         */
	FIN (ip_car_profiles_defaults_register ());

	/* Allocate memory to store the info about the current profile.	*/	
	profile_ptr = (OmsT_Qm_Car_Profile *) op_prg_mem_alloc (sizeof (OmsT_Qm_Car_Profile));

	/* Read the information from the attribute and fill the data structure	*/
	profile_ptr->name = (char *) op_prg_mem_alloc ((strlen("IP Limit") + 1) * sizeof (char));
	strcpy (profile_ptr->name, "All Traffic Limit");

	/* Determine the number of COS.	*/
	profile_ptr->number_of_cos = 1;

	/* Allocate memory for all the COS */
	profile_ptr->cos_definition = (OmsT_Qm_Cos_Definition *) op_prg_mem_alloc (sizeof (OmsT_Qm_Cos_Definition));
            
	profile_ptr->cos_definition[0].tos = -1;
	profile_ptr->cos_definition[0].application_type = -1;
	profile_ptr->cos_definition[0].incoming_port = -1;

	profile_ptr->cos_definition[0].average_rate = 2000000;
	profile_ptr->cos_definition[0].normal_burst_size = 1000000;
	profile_ptr->cos_definition[0].excess_burst_size = 2000000;

	profile_ptr->cos_definition[0].conform_precedence = -1;
	profile_ptr->cos_definition[0].conform_action = 1;
			
	profile_ptr->cos_definition[0].excess_precedence = -1;
	profile_ptr->cos_definition[0].excess_action = 2;		

	/* Finally register the data into network-wide database by		*/
	/* passing the pointer to the data structure. That makes this	*/
	/* information accesible for all other objects in the network,	*/
	/* whichever needs it; for instance all IP routers.				*/
	oms_data_def_entry_insert ("CAR Profiles", profile_ptr->name, profile_ptr); 


	/* Allocate memory to store the info about the current profile.	*/	
	profile_ptr = (OmsT_Qm_Car_Profile *) op_prg_mem_alloc (sizeof (OmsT_Qm_Car_Profile));

	/* Read the information from the attribute and fill the data structure	*/
	profile_ptr->name = (char *) op_prg_mem_alloc ((strlen("HTTP Limit") + 1) * sizeof (char));
	strcpy (profile_ptr->name, "HTTP Traffic Limit");

	/* Determine the number of COS.	*/
	profile_ptr->number_of_cos = 2;

	/* Enable the CAR profile.	*/
	profile_ptr->car_status = OPC_TRUE;

	/* Allocate memory for all the COS */
	profile_ptr->cos_definition = (OmsT_Qm_Cos_Definition *) op_prg_mem_alloc (2 * sizeof (OmsT_Qm_Cos_Definition));
            
	profile_ptr->cos_definition[0].tos = -1;
	profile_ptr->cos_definition[0].application_type = -1;
	profile_ptr->cos_definition[0].incoming_port = -1;

	profile_ptr->cos_definition[0].average_rate = 5000000;
	profile_ptr->cos_definition[0].normal_burst_size = 1000000;
	profile_ptr->cos_definition[0].excess_burst_size = 2000000;

	profile_ptr->cos_definition[0].conform_precedence = -1;
	profile_ptr->cos_definition[0].conform_action = 1;
			
	profile_ptr->cos_definition[0].excess_precedence = -1;
	profile_ptr->cos_definition[0].excess_action = 2;
	

	profile_ptr->cos_definition[1].tos = -1;
	profile_ptr->cos_definition[1].application_type = 80;
	profile_ptr->cos_definition[1].incoming_port = -1;

	profile_ptr->cos_definition[1].average_rate = 1000000;
	profile_ptr->cos_definition[1].normal_burst_size = 1000000;
	profile_ptr->cos_definition[1].excess_burst_size = 2000000;

	profile_ptr->cos_definition[1].conform_precedence = 3;
	profile_ptr->cos_definition[1].conform_action = 1;
			
	profile_ptr->cos_definition[1].excess_precedence = -1;
	profile_ptr->cos_definition[1].excess_action = 2;

	
	/* Finally register the data into network-wide database by		*/
	/* passing the pointer to the data structure. That makes this	*/
	/* information accesible for all other objects in the network,	*/
	/* whichever needs it; for instance all IP routers.				*/
	oms_data_def_entry_insert ("CAR Profiles", profile_ptr->name, profile_ptr);

	FOUT;
	}
