/* Opnet Model Support package		*/
/* IP Queue Management sub-package	*/
/* ip_qos_support.ex.c				*/

/****************************************/
/*  Copyright (c) 1987 - 1998		*/
/*            by MIL 3, Inc.           	*/
/*       (A Delaware Corporation)      	*/
/*    3400 International Drive, N.W.   	*/
/*       Washington, D.C., U.S.A.      	*/
/*       All Rights Reserved.          	*/
/****************************************/


/* Define a symbolic constant for export. */
#define OMSC_EXPORT		DLLEXPORT
#define					OmsC_Transmit	1
#define					OmsC_Drop		2
#define					OmsC_Any		-1
#define					OmsC_Unchanged	-1

/* ---- (SECTION_INC) Include Directives ---- 	*/
#include			<opnet.h>
#include			<ip_addr_v4.h>
#include			<ip_dgram_sup.h>
#include			<tcp_seg_sup.h>
#include			<udp_dgram_sup.h> 
#include			<oms_qm.h>
#include			<ip_qos_notif_log_support.h>
#include			<string.h>

/*  ----------- External Functions ------------------ */


OMSC_EXPORT Boolean
Ip_Qos_Car_Policy_Limit (Packet* pkptr, int incoming_interface, OmsT_Qm_Car_Profile* car_profile, OmsT_Qm_Car_Information* car_info)
	{
	IpT_Dgram_Fields*                   pk_fd_ptr;
	int									cos, tos, source_port, dest_port;
	int									packet_size, num_cos;
	Packet *                            pk_data_ptr;
	TcpT_Seg_Fields*                    tcp_fields_ptr;
	UdpT_Dgram_Fields*                  udp_fields_ptr;
	Boolean								traffic_exceed = OPC_FALSE;
	double								extended_burst_size, potential_actual_debt;
	static Boolean						fragmented_pk_warning = OPC_FALSE;

	/** This function applies the CAR (Commited Access Rate)	**/
	/** interface policy. It returns whether a packet should	**/
	/** be dropped or not.										**/
	/** CAR limits bandwidth to a specified subrate for each	**/
	/** class of service (COS). Classes of service are defined	**/
	/** on application basis, incoming port or TOS. Classes of	**/
	/** Service based on application requires no fragmentation.	**/
	FIN (Ip_Qos_Car_Policy_Limit (pkptr, incoming_interface, car_profile, car_info));

	/* Get the packet Field.	*/
	if (op_pk_nfd_access (pkptr, "fields", &pk_fd_ptr) == OPC_INT_INVALID)
		{
		op_sim_end ("Unable to get the packet field", "", "", "");
		}

	/* Get the packet size.	*/
	packet_size = op_pk_total_size_get (pkptr);

	/* Get the packet TOS.	*/
	tos = pk_fd_ptr->tos;

	/* Get the application type, by getting the source port	*/
	/* the destination port in the packet field. For		*/
	/* example, HTTP has the port 80.						*/
	ip_qos_application_type_get (pkptr, pk_fd_ptr, &source_port, &dest_port);

	/* Get the number of COS.	*/
	num_cos = car_profile->number_of_cos;

	/* Loop through all the COS policies and find a match.	*/
	for (cos = 0; cos < num_cos; cos ++)
		{
		/* Give a warning when the application type criterion is used	*/
		/* whereas the packet is fragmented. The warning is only given	*/
		/* given once.													*/
		if ((!fragmented_pk_warning) &&
			(car_profile->cos_definition [cos].application_type != OmsC_Any) &&
			(pk_fd_ptr->frag))
			{
			/* Write a warning about the unability for CAR to handle	*/
			/* fragmented packet.										*/
			carnl_protwarn_pk_fragmented (op_pk_id (pkptr), op_pk_tree_id (pkptr));

			/* The warning will appear only once.	*/	
			fragmented_pk_warning = OPC_TRUE;
			}
		
		if	(((tos == car_profile->cos_definition [cos].tos)
				|| (car_profile->cos_definition [cos].tos == OmsC_Any))
			&&
    		((incoming_interface == car_profile->cos_definition [cos].incoming_port)
				|| (car_profile->cos_definition [cos].incoming_port == OmsC_Any))
			&& 
			((source_port == car_profile->cos_definition [cos].application_type)
				|| (dest_port == car_profile->cos_definition [cos].application_type)
				|| (car_profile->cos_definition [cos].application_type == OmsC_Any)))
			{
			/* The packet belongs to a specified COS in the profile.	*/

			/* Update the token bucket for this COS.	*/
			car_info [cos].bucket_size = car_info [cos].bucket_size + car_profile->cos_definition [cos].average_rate *
										(op_sim_time () - car_info [cos].last_update_time);
		
			/* Store the time at which the token bucket	*/
			/* has been last updated.					*/
			car_info [cos].last_update_time = op_sim_time ();
			
			/* Update the potential debt for this flow.	*/
			if (car_info [cos].actual_debt > 0)
				{
				/* The flow has a debt. Tokens will first try to reduce this debt	*/
				/* before filling the bucket.										*/
				if (car_info [cos].bucket_size >= car_info [cos].actual_debt)
					{
					/* There are enough tokens to fullfill the debt.	*/
					car_info [cos].bucket_size -= car_info [cos].actual_debt;
					car_info [cos].actual_debt = 0;
					}
				else
					{
					/* The debt can be reduced.	*/
					car_info [cos].actual_debt -= car_info [cos].bucket_size;
					car_info [cos].bucket_size = 0;
					}
				}

			/* Drop some tokens if the bucket is full.	*/
			if (car_info [cos].bucket_size > car_profile->cos_definition [cos].normal_burst_size)
				{
				car_info [cos].bucket_size = car_profile->cos_definition [cos].normal_burst_size;
				}

			/* Check whether there are enough tokens in the bucket. If not, the flow	*/
			/* will borrow packets up to the extended burst value.						*/
			if (packet_size < car_info [cos].bucket_size)
				{
				/* There are enough tokens in the bucket.	*/	
				
				/* Update the bucket size.  */
				car_info [cos].bucket_size = car_info [cos].bucket_size - packet_size;
				
				/* The traffic is conforming.	*/
				traffic_exceed = OPC_FALSE;
				}
			else
				{
				/* There are not enough tokens in the bucket. The packet will try to	*/
				/* borrow some tokens up to the extended burst size.					*/

				/* Compute the extended burst size.	*/
				extended_burst_size = car_profile->cos_definition [cos].excess_burst_size -
										car_profile->cos_definition [cos].normal_burst_size;
				
				/* Compute the debt for this flow if it borrows some tokens.	*/
				potential_actual_debt = car_info [cos].actual_debt + packet_size - car_info [cos].bucket_size;

				if ((potential_actual_debt > extended_burst_size) || (car_info [cos].compounded_debt > extended_burst_size))
					{
					/* The packet is not allowed to borrow tokens	*/
					/* because its COS has already a big debt.		*/

					/* The coumpounded debt is set to zero.	*/
					car_info [cos].compounded_debt = 0;
					
					/* The packet exceeds the extended burst size.	*/
					traffic_exceed = OPC_TRUE;
					}
				else
					{
					/* The packet is allowed to borrow some tokens.	*/
					car_info [cos].bucket_size = 0;
					car_info [cos].actual_debt = potential_actual_debt;
					car_info [cos].compounded_debt += car_info [cos].actual_debt;
			
					/* The traffic doesn't exceed the extended burst size yet.	*/
					traffic_exceed = OPC_FALSE;
					}
				}

			/* Check whether this packet can be transmitted.	*/
			if (!traffic_exceed)
				{
				/* The packet is conforming.	*/

				/* Set the IP precedence as specified.	*/
				if (car_profile->cos_definition [cos].conform_precedence != OmsC_Unchanged)
					{
					/* Set the new IP precedence in the packet header.  */
					pk_fd_ptr->tos = car_profile->cos_definition [cos].conform_precedence;
					}

				/* Make the action as specified for conforming traffic.	*/
				if (car_profile->cos_definition [cos].conform_action == OmsC_Transmit)
					{
					/* The packet is transmitted.	*/
					FRET (OPC_FALSE);
					}
				}
			else
				{
				/* The packet is not conforming.	*/

                /* Set the IP precedence as specified.  */
				if (car_profile->cos_definition [cos].excess_precedence != OmsC_Unchanged)
					{
					/* Set the new IP precedence in the packet header.  */
					pk_fd_ptr->tos = car_profile->cos_definition [cos].excess_precedence;
					}

				/* Make the action as specified for non-conforming traffic.	*/
				if (car_profile->cos_definition [cos].excess_action == OmsC_Drop)
					{
					FRET (OPC_TRUE);
					}

                /* Make the action as specified for conforming traffic. */
				if (car_profile->cos_definition [cos].excess_action == OmsC_Transmit)
					{
					/* The packet is transmitted.   */
					FRET (OPC_FALSE);
					}
				}
			}
		}

	/* The packet does not match any COS.	*/
	FRET (OPC_FALSE);
	}

/* ----------------------------- Internal Functions ------------------------- */

OMSC_EXPORT OmsT_Qm_Info *
ip_qos_info_assign (int interface_row_index, double speed, OmsT_Qm_Info * qm_info)
	{
	int				q_index, queuing_scheme;
	Objid			my_id;
	Objid			ip_qos_information_objid, ip_qos_information_child_objid;
	Objid			ip_addr_information_objid, ip_addr_information_child_objid;
	double			total_weight;
	char			classification_scheme [255];
	int				buffer_size;
	double			v_t, a_rate;			

	/** This function initializes the structure OmsT_info		**/
	/** at the beginning. Global attributes such as the maximum	**/
	/** number of buffered packets are put in this structure.	**/
	/** It is in charge of creating the structure to store the	**/
	/** packets.												**/
	/** This function is IP specific.							**/
	FIN (ip_qos_info_assign (interface_row_index, speed, qm_info));

	/* Get the object id of the current queue.	*/ 
	my_id = op_id_self ();

	/* We need to get the service rate of the processor.				*/
	/* Put the service rate in the appropriate field of the structure	*/
	/* OmsT_Qm_Info. The service rate is actually the data rate			*/
	/* of the link connected to the transmitter.						*/
	qm_info->service_rate = speed;    

    /* Get the object id of the IP Interface table.    */
	op_ima_obj_attr_get (my_id, "ip addr info", &ip_addr_information_objid);
	ip_addr_information_child_objid = op_topo_child (ip_addr_information_objid, OPC_OBJTYPE_GENERIC, interface_row_index);

	/* We need to get the information about queuing classification.		*/
	/* Get the object id of the attribute called IP QoS Information.	*/
	op_ima_obj_attr_get (ip_addr_information_child_objid, "QoS info", &ip_qos_information_objid);

	/* Get the object id of the child of the compound attribute.	*/
	ip_qos_information_child_objid = 
		op_topo_child (ip_qos_information_objid, OPC_OBJTYPE_GENERIC, 0);

	/* Get the queuing classification of the attribute called "IP QoS Information".	*/
	op_ima_obj_attr_get (ip_qos_information_child_objid, "Queuing Profile", &classification_scheme);
	
	/* Get the queuing scheme: FIFO, WFQ, VC,DRR priority queuing, custom queuing or Virtual Clock.	*/
	op_ima_obj_attr_get (ip_qos_information_child_objid, "Queuing Scheme", &queuing_scheme);
		
	/* Get the buffer size in bytes of the interface.   */ 
	op_ima_obj_attr_get (ip_qos_information_child_objid, "Buffer Size", &buffer_size); 
	qm_info->max_buffer_size = (double) buffer_size;

	/* QoS uses a database where attributes are stored to save memory.	*/
	/* The category name depends on the queuing scheme.					*/
	switch (queuing_scheme)
		{
		case IpC_FIFO_Queuing:	
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("FIFO Profiles", classification_scheme);
			break;
			}
		case IpC_WFQ_Queuing:	
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("WFQ Profiles", classification_scheme);
							
			break;
			}
		case IpC_VC_Queuing:
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("VC Profiles", classification_scheme);
			break;
			}
		case IpC_DRR_Queuing:
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("DRR Profiles", classification_scheme);
			break;
			}
		case IpC_Priority_Queuing:
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("Priority Queuing Profiles", classification_scheme);
		  	break;
			}
		case IpC_Custom_Queuing:
			{
			qm_info->attributes_ptr = (OmsT_Qm_Attributes *) oms_data_def_entry_access ("Custom Queuing Profiles", classification_scheme);
			break;
			}	   
		default:
		   {
		   /* No queuing processing has been chosen.   */
		   op_sim_end ("No Queuing scheme has been chosen", "", "", "");
		   }
	   }

	/* Stop the simulation if the queuing scheme is not present in the database.	*/
	if (qm_info->attributes_ptr == OPC_NIL)
		{
		op_sim_end ("The entry name in the sub-attribute 'Queuing Profile' of 'IP QoS Information' doesn't exist.", "", "", "");		
		}

	/* Allocates memory for all the queues pointers. Pointers are needed when	*/
	/* using dynamic queues such as RSVP.										*/
	qm_info->queue_info_ptr = (OmsT_Qm_Queue_Info **) 
			op_prg_mem_alloc (qm_info->attributes_ptr->no_queues * sizeof (OmsT_Qm_Queue_Info*));

	/* Loop through all the queues and allocate memory for each queue.	*/
	for (q_index = 0; q_index < qm_info->attributes_ptr->no_queues; q_index ++)
		{
		qm_info->queue_info_ptr [q_index] = (OmsT_Qm_Queue_Info *) 
					op_prg_mem_alloc (qm_info->attributes_ptr->no_queues * sizeof (OmsT_Qm_Queue_Info));
		}

	/* Ends the simulation if the allocation fails.	*/
	if (qm_info->queue_info_ptr == OPC_NIL)
		{
		op_sim_end ("Unable to allocate memory for the queues.", "", "", "");
		}
	
	/* If queuing scheme is Virtual Clock Loop through all the queues to	*/
	/* initialize the virtual_clock and	 aux_virtual_clock variables.		*/
	if (queuing_scheme == IpC_VC_Queuing)
		{
		for (q_index = 0; q_index < qm_info->attributes_ptr->no_queues; q_index++)
			{ 
			qm_info->attributes_ptr->queue_configuration[q_index]->virtual_clock = 0.0;
			qm_info->attributes_ptr->queue_configuration[q_index]->aux_virtual_clock = 0.0;
			qm_info->attributes_ptr->queue_configuration[q_index]->first_packet_flag = OPC_FALSE;
			a_rate = qm_info->attributes_ptr->queue_configuration[q_index]->arrival_rate;
			v_t = 1 / a_rate;
			qm_info->attributes_ptr->queue_configuration[q_index]->curr_virtual_clock = v_t;
			}
		}
	
	
	
	/* if queuing scheme is Deficit Round Robin loop through all the queues to */
	/* initialize the deficint counter variable. 									   */
	if (queuing_scheme == IpC_DRR_Queuing)
		{
		for (q_index = 0; q_index < qm_info->attributes_ptr->no_queues; q_index++)
			{ 
   			qm_info->attributes_ptr->queue_configuration[q_index]->d_counter = 0.0;
   			}
		}
		

	if (queuing_scheme == IpC_WFQ_Queuing)
		{
		/* determine the total weight of the queues in order	*/
		/* to give a relative weight to each queue.				*/
		total_weight = 0;
     	for (q_index = 0; q_index < qm_info->attributes_ptr->no_queues; q_index ++)
			{
			/* Get the weight of the queue.	*/
			total_weight = total_weight + qm_info->attributes_ptr->queue_configuration[q_index]->weight;
			}

		/* Store the total weight. To use a relative weight we have to divide	*/
		/* the absolute weight of each queue by this value.						*/
		qm_info->total_weight = total_weight;
		}

	/* Fill in all the attributes of each queue such as the weight, the	*/
	/* ToS, the protocol...												*/
	for (q_index = 0; q_index < qm_info->attributes_ptr->no_queues; q_index++)
		{
		/* Create a empty list to store the packets.	*/
		qm_info->queue_info_ptr[q_index]->pk_list_ptr = op_prg_list_create ();

		/* Initialize the previous finish time to 0 second and	*/
		/* the finish time as uncomputed.						*/
		qm_info->queue_info_ptr[q_index]->previous_time = 0.0;
		qm_info->queue_info_ptr[q_index]->finish_time = OmsC_Uncomputed;
		qm_info->queue_info_ptr[q_index]->average_queue_size = 0.0;
		qm_info->queue_info_ptr[q_index]->start_idle_time = 0.0;
		qm_info->queue_info_ptr[q_index]->current_queue_size_in_bytes = 0.0;
		qm_info->queue_info_ptr[q_index]->queue_to_delete = OPC_FALSE;
		}

	/* Return the initialized structure.	*/				
	FRET (qm_info);
	}


OMSC_EXPORT OmsT_Qm_Info *
ip_qos_with_rsvp_info_assign (int interface_row_index, double speed, OmsT_Qm_Info * qm_info)
	{
	Objid									my_id;
	OmsT_Qm_Classification_Criteria *		classify_scheme;
	int										queuing_scheme;
	Objid									ip_addr_information_objid, ip_addr_information_child_objid;
	Objid									ip_qos_information_objid, ip_qos_information_child_objid;
	Objid									rsvp_info_objid, rsvp_info_child_objid;
	double									max_res_bandwidth;
	int										buffer_size;

	/** This function initializes the structure OmsT_info		**/
	/** at the beginning. Global attributes such as the maximum	**/
	/** number of buffered packets are put in this structure.	**/
	/** It is in charge of creating the structure to store the	**/
	/** packets for default traffic when RSVP is enabled.		**/
	/** This function is IP specific and will be called when	**/
	/** RSVP is enabled.										**/
	FIN (ip_qos_with_rsvp_info_assign (interface_row_index, speed, qm_info));

	/* Get the object id of the current queue.	*/ 
	my_id = op_id_self ();

   /* Get the object id of the IP Interface table.    */
   op_ima_obj_attr_get (my_id, "ip addr info", &ip_addr_information_objid);
   ip_addr_information_child_objid = op_topo_child (ip_addr_information_objid, OPC_OBJTYPE_GENERIC, interface_row_index);
			
	/* We need to get the information about queuing classification.     */
	/* Get the object id of the attribute called IP QoS Information.    */
	op_ima_obj_attr_get (ip_addr_information_child_objid, "QoS info", &ip_qos_information_objid);
						 
	/* Get the object id of the child of the compound attribute.    */
	ip_qos_information_child_objid =
	op_topo_child (ip_qos_information_objid, OPC_OBJTYPE_GENERIC, 0);
										  
	/* Get the queuing scheme: FIFO, WFQ, VC, DRR, priority queuing or custom queuing.   */
	op_ima_obj_attr_get (ip_qos_information_child_objid, "Queuing Scheme", &queuing_scheme);

	/* Get the buffer size in bytes of the interface.   */
	op_ima_obj_attr_get (ip_qos_information_child_objid, "Buffer Size", &(buffer_size));
	qm_info->max_buffer_size = (double) buffer_size;

    /* Get the RSVP parameters of the interface.   */
	op_ima_obj_attr_get (ip_qos_information_child_objid, "RSVP Info", &(rsvp_info_objid));
	rsvp_info_child_objid = op_topo_child (rsvp_info_objid, OPC_OBJTYPE_GENERIC, 0);

	/* Get the maximum reservable bandwidth.	*/
	op_ima_obj_attr_get (rsvp_info_child_objid, "Max Reservable Bandwidth", &max_res_bandwidth);

	/* We need to get the service rate of the processor.				*/
	/* Put the service rate in the appropriate field of the structure	*/
	/* OmsT_Qm_Info. The service rate is actually the data rate			*/
	/* of the link connected to the transmitter.						*/
	qm_info->service_rate = speed;    	   

	/* Allocate memory to store the info about the current scheme.	*/	
	qm_info->attributes_ptr = (OmsT_Qm_Attributes *) op_prg_mem_alloc (sizeof (OmsT_Qm_Attributes));

	/* This is the default RSVP queue.	*/
	qm_info->attributes_ptr->name = (char *) op_prg_mem_alloc ((strlen("RSVP queue") + 1) * sizeof (char));
	strcpy (qm_info->attributes_ptr->name, "RSVP queue");

	/* Set some parameters for the default queue.	*/
	qm_info->attributes_ptr->no_queues 				= 1;
	qm_info->attributes_ptr->max_total_no_buf_pkts 	= 1;

	/* Allocate memory for the configuration of the queue.	*/
	qm_info->attributes_ptr->queue_configuration = (OmsT_Qm_Queue_Configuration **) 
		op_prg_mem_alloc (sizeof (OmsT_Qm_Queue_Configuration *));

	qm_info->attributes_ptr->queue_configuration [0] = (OmsT_Qm_Queue_Configuration *) 
		op_prg_mem_alloc (sizeof (OmsT_Qm_Queue_Configuration));

	/* Initialize the queue configuration of the new element.	*/
	qm_info->attributes_ptr->queue_configuration [0]->weight 			= 100.0 - max_res_bandwidth;
	qm_info->attributes_ptr->queue_configuration [0]->byte_count_number = (int) ((100.0 - max_res_bandwidth) * OmsC_Rsvp_Total_Byte_Count / 100);
   	qm_info->attributes_ptr->queue_configuration [0]->max_size 			= (int) ((100.0 - max_res_bandwidth) * qm_info->max_buffer_size / 100);
   	qm_info->attributes_ptr->queue_configuration [0]->red_status 		= OmsC_No_Red;

	 /* Create a list to store the classification schemes.	*/
	 qm_info->attributes_ptr->classification_list_ptr = op_prg_list_create ();

	/* Allocate memory for all the criteria available for this queue.	*/
	classify_scheme = (OmsT_Qm_Classification_Criteria *) op_prg_mem_alloc (sizeof (OmsT_Qm_Classification_Criteria));

	/* Set the classification parameters for the default queue.	*/
	classify_scheme->protocol 		= OmsC_Unassigned;
	classify_scheme->tos 			= OmsC_Unassigned;
	classify_scheme->source_address = IPC_ADDR_INVALID;
   	classify_scheme->dest_address 	= IPC_ADDR_INVALID;
	classify_scheme->source_port 	= OmsC_Unassigned;
	classify_scheme->dest_port 		= OmsC_Unassigned;
	classify_scheme->incoming_iface = OmsC_Unassigned;
	classify_scheme->queue_id 		= 0;

	/* Insert this set of criteria with the queue id in a list.	*/
	/* This list contains all the information to enqueue a 		*/
	/* packet in a specific queue. If there is no match in this	*/
	/* list the packet will be put in the default queue.		*/
	op_prg_list_insert (qm_info->attributes_ptr->classification_list_ptr, classify_scheme, OPC_LISTPOS_TAIL);

	/* Allocates memory for all the queues pointers. Pointers are needed when	*/
	/* using dynamic queues such as RSVP.										*/
	qm_info->queue_info_ptr = (OmsT_Qm_Queue_Info **) 
			op_prg_mem_alloc (sizeof (OmsT_Qm_Queue_Info*));

	qm_info->queue_info_ptr [0] = (OmsT_Qm_Queue_Info *) 
	    	op_prg_mem_alloc (qm_info->attributes_ptr->no_queues * sizeof (OmsT_Qm_Queue_Info));

	/* Ends the simulation if the allocation fails.	*/
	if (qm_info->queue_info_ptr == OPC_NIL)
		{
		op_sim_end ("Unable to allocate memory for the queues.", "", "", "");
		}

	/* Create a empty list to store the packets.	*/
	qm_info->queue_info_ptr[0]->pk_list_ptr = op_prg_list_create ();

	/* Initialize the previous finish time to 0 second and	*/
	/* the finish time as uncomputed.						*/
	qm_info->queue_info_ptr[0]->previous_time 				= 0.0;
	qm_info->queue_info_ptr[0]->finish_time 				= OmsC_Uncomputed;
	qm_info->queue_info_ptr[0]->average_queue_size 			= 0.0;
	qm_info->queue_info_ptr[0]->start_idle_time 			= 0.0;
	qm_info->queue_info_ptr[0]->current_queue_size_in_bytes	= 0.0;
	qm_info->queue_info_ptr[0]->queue_to_delete				= OPC_FALSE;
	/* Return the initialized structure.	*/				
	FRET (qm_info);
	}

OMSC_EXPORT OmsT_Qm_Pkt_Info
ip_qos_tos_classifier_obtain (Packet* pkptr, OmsT_Qm_Info* qm_info, Boolean* ip_warning_message_print_ptr)
	{
	int									queue_index, list_index;
	OmsT_Qm_Pkt_Info					pkt_info;	
	IpT_Dgram_Fields*					pk_fd_ptr;
	Packet *							pk_data_ptr;
	TcpT_Seg_Fields*					tcp_fields_ptr;
	UdpT_Dgram_Fields*					udp_fields_ptr;
	OmsT_Qm_Classification_Criteria*	classify_info_ptr;
	Ici*								ici_ptr;

	/** This function get the criteria needed to confer a			**/
	/** priority to the incoming packet. The criteria is the ToS.	**/
	/** This will allow the processor to enqueue the packet into	**/
	/** the right weighted	queue.									**/
	/** All these information are returned in a structure.			**/
	/** This function is specific to IP.							**/
	FIN (ip_qos_tos_classifier_obtain (pkptr, qm_info, warning_message));

	/* Get the content of the Field called "fields" in the IP-Datagram			*/
	/* in order to get the tos, protocol, the destination and source address.	*/
    if (op_pk_nfd_access (pkptr, "fields", &pk_fd_ptr) == OPC_COMPCODE_FAILURE) 
		{
		pkt_info.tos 			= OmsC_Unassigned; 
		pkt_info.protocol 		= OmsC_Unassigned;   				    	
		pkt_info.source_port 	= OmsC_Unassigned;
		pkt_info.dest_port 		= OmsC_Unassigned;
		pkt_info.incoming_iface	= OmsC_Unassigned;
		pkt_info.source_address = IPC_ADDR_INVALID;
		pkt_info.dest_address = IPC_ADDR_INVALID;
		}
	else
		{
		/* Get the ToS, protocol, IP source and destination 	*/
		/* addresses from the datagram.		   					*/	   
		pkt_info.tos 	  = pk_fd_ptr->tos; 
		pkt_info.protocol = pk_fd_ptr->protocol;   				    	

		/* Get the source and destination IP addresses.	*/
		pkt_info.source_address = pk_fd_ptr->src_addr;
		pkt_info.dest_address = pk_fd_ptr->dest_addr;
	  
		/* Get the incoming port from the ICI.	*/
		ici_ptr = op_pk_ici_get (pkptr);
		op_ici_attr_get (ici_ptr, "incoming_iface", &pkt_info.incoming_iface);

		/* Get the application type by getting the source port	*/
		/* and the destination port in the packet field. For	*/
		/* example, HTTP has the port 80.                       */
		ip_qos_application_type_get (pkptr, pk_fd_ptr, &pkt_info.source_port, &pkt_info.dest_port);
		}		

	/* Assign list index to the indice of the last element of the list.	*/
	list_index = op_prg_list_size (qm_info->attributes_ptr->classification_list_ptr) -1;

	/* Determine the queue in which the packet should be		*/
	/* according to its tos, protocol, source and destination	*/
	/* addresses, source and destination ports.					*/
	while (list_index >= 0)
	    {
	 	/* Access one element of the list of criteria. */
	 	classify_info_ptr = (OmsT_Qm_Classification_Criteria *) 
			op_prg_list_access (qm_info->attributes_ptr->classification_list_ptr, list_index);

	    if ((((pkt_info.tos != classify_info_ptr->tos) &&
			(classify_info_ptr->tos != OmsC_Unassigned))
		|| ((pkt_info.protocol != classify_info_ptr->protocol) &&
			(classify_info_ptr->protocol != OmsC_Unassigned))
		|| ((pkt_info.source_address != classify_info_ptr->source_address) &&
			(classify_info_ptr->source_address != IPC_ADDR_INVALID))
		|| ((pkt_info.dest_address != classify_info_ptr->dest_address) &&
			(classify_info_ptr->dest_address != IPC_ADDR_INVALID))
		|| ((pkt_info.source_port != classify_info_ptr->source_port) &&
			(classify_info_ptr->source_port != OmsC_Unassigned))
		|| ((pkt_info.dest_port != classify_info_ptr->dest_port) &&
			(classify_info_ptr->dest_port != OmsC_Unassigned))
		|| ((pkt_info.incoming_iface != classify_info_ptr->incoming_iface)	&&
			(classify_info_ptr->incoming_iface != OmsC_Unassigned)))
		|| ((classify_info_ptr->tos == OmsC_Unassigned) &&
			(classify_info_ptr->protocol == OmsC_Unassigned) &&
		   	(classify_info_ptr->source_address == IPC_ADDR_INVALID) &&
			(classify_info_ptr->dest_address == IPC_ADDR_INVALID) &&
		  	(classify_info_ptr->source_port == OmsC_Unassigned) &&
			(classify_info_ptr->dest_port == OmsC_Unassigned)	&&
			(classify_info_ptr->incoming_iface == OmsC_Unassigned)))	 
			{
			if (!(* ip_warning_message_print_ptr))	
				{
				/* Print a notification message to warn that a packet can not	*/
				/* be put into the right queue according to the tcp/udp port	*/
				/* because the packet is a fragment.							*/
				if ((pk_fd_ptr->frag) 
				&& ((classify_info_ptr->source_port != OmsC_Unassigned)
					|| (classify_info_ptr->dest_port != OmsC_Unassigned)))
					{	
					wfqnl_protwarn_missing_port (op_pk_id (pkptr), op_pk_tree_id (pkptr));
					* ip_warning_message_print_ptr = OPC_TRUE;
					}
				}
			list_index --;
			}
	    else
			{
			/* A queue matches the criteria.	*/
			queue_index = classify_info_ptr->queue_id;
			break;
			}
	    }

	/* By default if there is no appropriate queue for the	*/
	/* packet it is put in the queue 0.						*/
	if (list_index < 0)
		queue_index = 0;

	/* Memorizes the index of the queue in which the packet	*/
	/* should be put according to its tos, protocol...		*/
	pkt_info.queue_id = queue_index;	

	/* Return the informations contained in the packet.	*/
	FRET (pkt_info);
	}


OMSC_EXPORT OmsT_Qm_Pkt_Info
ip_qos_fifo_classifier_obtain (Packet * pkptr, OmsT_Qm_Info * qm_info, Boolean * ip_warning_message_print_ptr)
	{
	OmsT_Qm_Pkt_Info        pkt_info;

	/** This function get the criteria needed to confer a priority	**/
	/** to the incoming packet. In FIFO, there are no criteria 		**/
	/** since they is only one queue. Returns this queue id.		**/
	FIN (ip_qos_fifo_classifier_obtain (pkptr, qm_info, warning_message));

	pkt_info.queue_id = 0;

	FRET (pkt_info);
	}


OMSC_EXPORT void
ip_qos_application_type_get (Packet* pkptr, IpT_Dgram_Fields* pk_fd_ptr, int *source_port, int *dest_port)
	{
	TcpT_Seg_Fields*                    tcp_fields_ptr;
	UdpT_Dgram_Fields*                  udp_fields_ptr;
	Packet *                            pk_data_ptr;

	/** This function returns the source and destination application	**/
	/** type in the TCP/UDP header of a IP datagram. This application	**/
	/** type is used for instance by CAR, WFQ, Custom Queuing...		**/
	FIN (ip_qos_applicaiton_type_get (pkptr, pk_fd_fields*, source_port_dest_port));

	/* Get the TCP  or UDP port if the packet is not a fragment.	*/
	/* A router is not able to re-fragmente an IP frame. If the		*/
	/* packet is not fragmented it can find the TCP/UDP port in		*/
	/* the TCP header encapsulated in the data of the IP datagram.	*/
	
	/* Check whether the packet is fragmented. If the packet is		*/
	/* fragmented, then the application type criterion won't		*/
	/* apply.														*/
	if (!pk_fd_ptr->frag)
		{ 
 	  	/* check if there are some data in the IP-datagram.	*/
   	  	if (op_pk_nfd_is_set (pkptr, "data"))
			 {
			 /* Get the data from the IP-datagram.	*/
			 if (op_pk_nfd_get (pkptr, "data", &pk_data_ptr) == OPC_COMPCODE_FAILURE)
			    {
			    /* Stop the simuation if there is no data in the packet.	*/
				op_sim_end ("Unable to get data from packet.", "", "", "");
			    }
				
			/* Check if the transport protocol is TCP	*/
			/* to acquire the TCP  port number.			*/
			if (pk_fd_ptr->protocol == IpC_Protocol_Tcp)
				{  
				/* Get the fields structure from the TCP header.*/
				op_pk_nfd_access (pk_data_ptr, "fields", &tcp_fields_ptr); 
		
				/* Get the source and destination TCP ports	*/
				/* from the fields called fields in the TCP	*/
				/* header.									*/
				*source_port = tcp_fields_ptr->src_port;
				*dest_port = tcp_fields_ptr->dest_port; 
				}
			else
				{
				/* Check if the transport protocol is UDP	*/
	   		    /* to acquire the UDP port number.			*/
				if (pk_fd_ptr->protocol == IpC_Protocol_Udp)
					{   
					/* Get the fields structure from the UDP header.	*/
					op_pk_nfd_access (pk_data_ptr, "fields", &udp_fields_ptr);   
	
					/* Get the source and destination TCP ports	*/
					/* from the fields called fields in the TCP	*/
					/* header.									*/
					*source_port = udp_fields_ptr->src_port;
					*dest_port = udp_fields_ptr->dest_port;
					} 
				else
					{
					/* If the transport protocol is not TCP or UDP	*/
					/* the ports are considered as unassigned.		*/
    				*source_port = OmsC_Any;
					*dest_port = OmsC_Any;
					}
				}
			/* Put the data in the datagram after use.	*/
			op_pk_nfd_set (pkptr, "data", pk_data_ptr);  
			}
		} 			    	
	else
		{
		/* If the packet is a fragment it is not possible to get the TCP or UDP port.	*/
		*source_port = OmsC_Any;
		*dest_port = OmsC_Any;
		}

	FOUT;
	}



